
{% load widget_tweaks %}


<div class="col-span-12">
  <div class="form-control w-full">
    <label class="label">
      <span class="label-text">{{ field.label | safe}}</span>
    </label>

    {% render_field field class=field_input_class class+="input input-bordered w-full" %}
    {% if field.help_text %}
      <label class="label">
        <span class="label-text-alt">{{field.help_text}}</span>
      </label>
    {% endif %}
  </div>
</div>
