# Generated by Django 4.2.20 on 2025-03-25 10:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slack", "0002_usergroup_tag"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usergroup",
            name="tag",
            field=models.CharField(
                blank=True,
                help_text="Used by FireFighter internally to mark special user groups (e.g. @team-secu, @team-incidents...). Must be empty or unique.",
                max_length=80,
            ),
        ),
    ]
