# Generated by Django 4.2.21 on 2025-10-13 13:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0028_add_closure_reason_constraint"),
    ]

    operations = [
        migrations.AddField(
            model_name="incident",
            name="custom_fields",
            field=models.JSONField(
                blank=True,
                default=dict,
                help_text="Custom fields for incident (zendesk_ticket_id, seller_contract_id, etc.)",
            ),
        ),
    ]
