# Generated by Django 4.2.23 on 2025-09-17 17:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("raid", "0002_featureteam_remove_qualifierrotation_jira_user_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="RaidArea",
        ),
    ]
