# Generated by Django 4.2.16 on 2024-10-14 10:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slack", "0001_initial_oss"),
    ]

    operations = [
        migrations.AddField(
            model_name="usergroup",
            name="tag",
            field=models.CharField(
                blank=True,
                help_text="Used by FireFighter internally to mark special users group (@team-secu, @team-incidents ...). Must be empty or unique.",
                max_length=80,
            ),
        ),
    ]
