# Generated by Django 4.2.11 on 2024-05-03 09:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0002_alter_severity_name_alter_user_password_featureteam"),
    ]

    operations = [
        migrations.DeleteModel(
            name="FeatureTeam",
        ),
    ]
