# Generated by Django 4.2.14 on 2024-09-27 10:06

from django.db import migrations, models

import firefighter.incidents.models.environment


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0003_delete_featureteam"),
    ]

    operations = [
        migrations.AddField(
            model_name="incidentupdate",
            name="environment",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=models.SET(
                    firefighter.incidents.models.environment.Environment.get_default
                ),
                to="incidents.environment",
            ),
        ),
    ]
