# Generated by Django 3.2.8 on 2021-11-07 16:58

from __future__ import annotations

from django.db import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("authtoken", "0003_tokenproxy"),
    ]

    operations = [
        migrations.CreateModel(
            name="APIToken",
            fields=[],
            options={
                "proxy": True,
                "default_permissions": [],
                "indexes": [],
                "constraints": [],
            },
            bases=("authtoken.token",),
        ),
        migrations.CreateModel(
            name="APITokenProxy",
            fields=[],
            options={
                "verbose_name": "API Token",
                "permissions": (
                    ("can_edit_any", "Can reassign token to any user"),
                    ("can_add_any", "Can add token to any user"),
                    ("can_view_any", "Can view token of all users"),
                    ("can_delete_any", "Can delete token of any user"),
                    ("can_add_own", "Can add own tokens"),
                    ("can_view_own", "Can view own tokens"),
                    ("can_delete_own", "Can delete own tokens"),
                ),
                "abstract": False,
                "proxy": True,
                "default_permissions": [],
                "indexes": [],
                "constraints": [],
            },
            bases=("api.apitoken",),
        ),
    ]
