# Generated by Django 4.2.23 on 2025-09-17 17:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0026_alter_incidentcategory_options_and_more"),
        ("slack", "0007_remove_components_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="conversation",
            name="incident_categories",
            field=models.ManyToManyField(
                blank=True,
                related_name="conversations",
                to="incidents.incidentcategory",
            ),
        ),
        migrations.AlterField(
            model_name="usergroup",
            name="incident_categories",
            field=models.ManyToManyField(
                blank=True,
                help_text="Incident created with this usergroup automatically add the group members to these issue categories.",
                related_name="usergroups",
                to="incidents.incidentcategory",
            ),
        ),
    ]
