{% component "modal" autoplay=True header=False %}

  {% fill "modal_header" %}
    Trigger on-call
  {% endfill %}
  {% fill "modal_content" %}

    <div class="max-w-7xl mx-auto px-4 sm:px-6 lg:px-8">
      <div role="alert" class="alert alert-warning shadow-lg">
        <svg xmlns="http://www.w3.org/2000/svg" class="stroke-current shrink-0 h-6 w-6" fill="none" viewBox="0 0 24 24"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 9v2m0 4h.01m-6.938 4h13.856c1.54 0 2.502-1.667 1.732-3L13.732 4c-.77-1.333-2.694-1.333-3.464 0L3.34 16c-.77 1.333.192 3 1.732 3z" /></svg>
        <span>Trigger on-call only if you feel it is necessary for the stability of our services.</span>
      </div>
      {% include "./trigger_oncall_form_view.html" %}
    </div>

  {% endfill %}
  {% fill "modal_enabler" %}
  {% endfill %}
{% endcomponent %}
