# Generated by Django 4.2.21 on 2025-06-03 15:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0014_update_components_slack_groups"),
        ("slack", "0003_alter_usergroup_tag"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usergroup",
            name="components",
            field=models.ManyToManyField(
                blank=True,
                help_text="Incident created with this usergroup automatically add the group members to these issue categories.",
                related_name="usergroups",
                to="incidents.component",
            ),
        ),
    ]
