# Generated by Django 4.2.21 on 2025-12-19 14:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0029_add_custom_fields_to_incident"),
    ]

    operations = [
        migrations.AddField(
            model_name="incident",
            name="mitigated_at",
            field=models.DateTimeField(
                blank=True,
                help_text="Timestamp when incident status changed to MITIGATED",
                null=True,
            ),
        ),
    ]
