<div class="mt-10">
  <form method="post" action="{% url 'pagerduty:oncall_trigger' %}">
    {% csrf_token %}
    {% component "form" form=form %}
      {% fill "form_footer" %}
        <div class="text-right sm:pl-6 py-8">
          <button type="submit" action="{% url 'pagerduty:oncall_trigger' %}" class="btn btn-primary">
            Trigger on-call
          </button>
        </div>
      {% endfill %}
    {% endcomponent %}
  </form>
</div>
