# Generated by Django 4.2.23 on 2025-09-26 12:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0027_add_closure_fields"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="incident",
            constraint=models.CheckConstraint(
                check=models.Q(
                    (
                        "closure_reason__in",
                        [
                            "resolved",
                            "duplicate",
                            "false_positive",
                            "superseded",
                            "external",
                            "cancelled",
                            None,
                        ],
                    )
                ),
                name="incidents_incident_closure_reason_valid",
            ),
        ),
    ]
