# Generated by Django 4.2.23 on 2025-09-17 17:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0025_make_incident_category_required"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="incidentcategory",
            options={
                "ordering": ["order"],
                "verbose_name_plural": "incident categories",
            },
        ),
        migrations.AlterField(
            model_name="impactlevel",
            name="description",
            field=models.TextField(
                blank=True,
                help_text="Detailed multi-line description for this impact level.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="impactlevel",
            name="emoji",
            field=models.CharField(default="▶", max_length=5),
        ),
        migrations.AlterField(
            model_name="impacttype",
            name="emoji",
            field=models.CharField(default="▶", max_length=5),
        ),
    ]
